@@ -19,50 +19,38 @@
 
 """
 
-from .Pickling import getStreamedConstant
+import ctypes
+import re
+import struct
 
-from .TupleCodes import addMakeTupleUse
-from .ListCodes import addMakeListUse
-from .DictCodes import addMakeDictUse
-from .SetCodes import addMakeSetUse
+import marshal
+from nuitka.__past__ import iterItems, long, unicode  # pylint: disable=W0622
+from nuitka.Constants import (
+    constant_builtin_types,
+    getConstantWeight,
+    isMutable
+)
 
 from .BlobCodes import StreamData
+from .Emission import SourceCodeCollector
+from .Pickling import getStreamedConstant
 
-from .Identifiers import (
-    EmptyDictIdentifier,
-    Identifier
-)
-# pylint: disable=W0622
-from ..__past__ import unicode, long, iterItems
-# pylint: enable=W0622
-
-from ..Constants import constant_builtin_types, isMutable, compareConstants
-
-import re, struct
-
-stream_data = StreamData("constant_bin")
-
-def getConstantHandle(context, constant):
-    return context.getConstantHandle(
-        constant = constant
-    )
+stream_data = StreamData()
 
 def getConstantCode(context, constant):
-    constant_identifier = context.getConstantHandle(
-        constant = constant
-    )
-
-    return constant_identifier.getCode()
+    return context.getConstantCode(constant)
 
 # TODO: The determination of this should already happen in Building or in a
 # helper not during code generation.
 _match_attribute_names = re.compile( r"[a-zA-Z_][a-zA-Z0-9_]*$" )
 
 def getConstantCodeName(context, constant):
-    return context.getConstantHandle( constant, real_use = False ).getCode()
+    return context.getConstantCode(constant)
+
 
 def _isAttributeName(value):
-    return _match_attribute_names.match( value )
+    return _match_attribute_names.match(value)
+
 
 # Indicator to standalone mode code, if we need pickling module early on.
 _needs_pickle = False
@@ -70,27 +58,32 @@ _needs_pickle = False
 def needsPickleInit():
     return _needs_pickle
 
-def _getUnstreamCode(constant_value, constant_identifier):
+
+def _getUnstreamCode2(constant_value):
     saved = getStreamedConstant(
         constant_value = constant_value
     )
 
-    assert type( saved ) is bytes
+    assert type(saved) is bytes
 
     # We need to remember having to use pickle, pylint: disable=W0603
     global _needs_pickle
     _needs_pickle = True
 
+    return stream_data.getStreamDataCode(saved)
+
+
+def _getUnstreamCode(constant_value, constant_identifier):
     return "%s = UNSTREAM_CONSTANT( %s );" % (
         constant_identifier,
-        stream_data.getStreamDataCode( saved )
+        _getUnstreamCode2(constant_value)
     )
 
-def _packFloat(value):
-    return struct.pack( "<d", value )
 
-import ctypes
-sizeof_long = ctypes.sizeof( ctypes.c_long )
+def _packFloat(value):
+    return struct.pack("<d", value)
+
+sizeof_long = ctypes.sizeof(ctypes.c_long)
 
 max_unsigned_long = 2**(sizeof_long*8)-1
 
@@ -98,10 +91,106 @@ max_unsigned_long = 2**(sizeof_long*8)-1
 # seems to not work (without warning) as literal, so avoid it.
 min_signed_long = -(2**(sizeof_long*8-1)-1)
 
+done = set()
+
+def _getConstantInitValueCode(context, constant_value, constant_type):
+    if constant_type is unicode:
+        try:
+            encoded = constant_value.encode("utf-8")
+
+            if str is not unicode:
+                return "UNSTREAM_UNICODE( %s )" % (
+                    stream_data.getStreamDataCode(encoded)
+                )
+            else:
+                return "UNSTREAM_STRING( %s, %d )" % (
+                    stream_data.getStreamDataCode(encoded),
+                    1 if _isAttributeName(constant_value) else 0
+                )
+        except UnicodeEncodeError:
+            return None
+    elif constant_type is str:
+        # Python3: Strings that can be encoded as UTF-8 are done more or less
+        # directly. When they cannot be expressed as UTF-8, that is rare not we
+        # can indeed use pickling.
+        assert str is not unicode
+
+        if len(constant_value) == 1:
+            return "UNSTREAM_CHAR( %d, %d )" % (
+                ord(constant_value[0]),
+                1 if _isAttributeName(constant_value) else 0
+            )
+        else:
+            return "UNSTREAM_STRING( %s, %d )" % (
+                stream_data.getStreamDataCode(constant_value),
+                1 if _isAttributeName(constant_value) else 0
+            )
+    elif constant_type is bytes:
+        assert str is unicode
+
+        return "UNSTREAM_BYTES( %s )" % (
+            stream_data.getStreamDataCode(constant_value)
+        )
+
+
+def isMarshalConstant(constant_value):
+    if getConstantWeight(constant_value) < 20:
+        return False
+
+    marshal_value = marshal.dumps(constant_value)
+    restored = marshal.loads(marshal_value)
+
+    return constant_value == restored
+
+
+def attemptToMarshal(constant_identifier, constant_value, emit):
+    # Only do it for sufficiently large constants, typically tuples of 20
+    # elements, or dicts of more than 10.
+    if getConstantWeight(constant_value) < 20:
+        return False
+
+    marshal_value = marshal.dumps(constant_value)
+    restored = marshal.loads(marshal_value)
+
+    if constant_value != restored:
+        return False
+
+    emit(
+        """%s = PyMarshal_ReadObjectFromString( (char *)%s );""" % (
+            constant_identifier,
+            stream_data.getStreamDataCode(marshal_value)
+        )
+    )
+
+    return True
+
+
 def _addConstantInitCode(context, emit, constant_type, constant_value,
-                         constant_identifier):
+                         constant_identifier, module_level):
     # This has many cases, that all return, and do a lot.
-    #  pylint: disable=R0911,R0912,R0915
+    # pylint: disable=R0911,R0912,R0915
+
+    if constant_value in constant_builtin_types:
+        return
+    if constant_value is None:
+        return
+    if constant_value is False:
+        return
+    if constant_value is True:
+        return
+    if constant_value is Ellipsis:
+        return
+    if constant_identifier in done:
+        return
+
+    if module_level:
+        if context.global_context.getConstantUseCount(constant_identifier) != 1:
+            return
+    else:
+        if context.getConstantUseCount(constant_identifier) == 1:
+            return
+
+    done.add(constant_identifier)
 
     # Use shortest code for ints and longs.
     if constant_type is long:
@@ -215,7 +304,7 @@ def _addConstantInitCode(context, emit, constant_type, constant_value,
         emit(
             "%s = UNSTREAM_BYTES( %s );" % (
                 constant_identifier,
-                stream_data.getStreamDataCode( constant_value )
+                stream_data.getStreamDataCode(constant_value)
             )
         )
 
@@ -226,7 +315,7 @@ def _addConstantInitCode(context, emit, constant_type, constant_value,
             "%s = UNSTREAM_FLOAT( %s );" % (
                 constant_identifier,
                 stream_data.getStreamDataCode(
-                    value      = _packFloat( constant_value ),
+                    value      = _packFloat(constant_value),
                     fixed_size = True
                 )
             )
@@ -234,321 +323,224 @@ def _addConstantInitCode(context, emit, constant_type, constant_value,
 
         return
 
-    if constant_value is None:
-        return
-
-    if constant_value is False:
-        return
-
-    if constant_value is True:
-        return
-
-    if constant_value is Ellipsis:
-        return
-
     if constant_type is dict:
-        if constant_value == {}:
-            emit( "%s = PyDict_New();" % constant_identifier )
-        else:
-            length = len( constant_value )
-            addMakeDictUse( length )
+        if attemptToMarshal(constant_identifier, constant_value, emit):
+            return
 
-            for key, value in iterItems( constant_value ):
-                _addConstantInitCode(
-                    emit                = emit,
-                    constant_type       = type( key ),
-                    constant_value      = key,
-                    constant_identifier = getConstantCodeName( context, key ),
-                    context             = context
-                )
-                _addConstantInitCode(
-                    emit                = emit,
-                    constant_type       = type( value ),
-                    constant_value      = value,
-                    constant_identifier = getConstantCodeName( context, value ),
-                    context             = context
-                )
+        emit(
+            "%s = _PyDict_NewPresized( %d );" % (
+                constant_identifier,
+                len(constant_value)
+            )
+        )
+
+        for key, value in iterItems(constant_value):
+            key_name = getConstantCodeName(context, key)
+            _addConstantInitCode(
+                emit                = emit,
+                constant_type       = type(key),
+                constant_value      = key,
+                constant_identifier = key_name,
+                module_level        = module_level,
+                context             = context
+            )
+
+            value_name = getConstantCodeName(context, value)
+            _addConstantInitCode(
+                emit                = emit,
+                constant_type       = type(value),
+                constant_value      = value,
+                constant_identifier = value_name,
+                module_level        = module_level,
+                context             = context
+            )
 
             emit(
-                "%s = MAKE_DICT%d( %s );" % (
+                "PyDict_SetItem( %s, %s, %s );" % (
                     constant_identifier,
-                    length,
-                    ", ".join(
-                        "%s, %s" % (
-                            getConstantCodeName( context, value ),
-                            getConstantCodeName( context, key )
-                        )
-                        for key, value
-                        in
-                        iterItems( constant_value )
-                    )
+                    key_name,
+                    value_name
                 )
             )
 
         return
 
     if constant_type is tuple:
-        if constant_value == ():
-            emit( "%s = PyTuple_New( 0 );" % constant_identifier )
-        else:
-            length = len( constant_value )
-            addMakeTupleUse( length )
+        if attemptToMarshal(constant_identifier, constant_value, emit):
+            return
 
-            # Make elements earlier than tuple itself.
-            for element in constant_value:
-                _addConstantInitCode(
-                    emit                = emit,
-                    constant_type       = type( element ),
-                    constant_value      = element,
-                    constant_identifier = getConstantCodeName(
-                        context  = context,
-                        constant = element
-                    ),
-                    context             = context
-                )
+        emit(
+            "%s = PyTuple_New( %d );" % (
+                constant_identifier,
+                len(constant_value)
+            )
+        )
 
+        for count, element_value in enumerate(constant_value):
+            element_name = getConstantCodeName(
+                context  = context,
+                constant = element_value
+            )
+
+            _addConstantInitCode(
+                emit                = emit,
+                constant_type       = type(element_value),
+                constant_value      = element_value,
+                constant_identifier = getConstantCodeName(
+                    context  = context,
+                    constant = element_value
+                ),
+                module_level        = module_level,
+                context             = context
+            )
+
+            # Do not take references, these won't be deleted ever.
             emit(
-                "%s = MAKE_TUPLE%d( %s );" % (
+                "PyTuple_SET_ITEM( %s, %d, %s ); Py_INCREF( %s );" % (
                     constant_identifier,
-                    length,
-                    ", ".join(
-                        getConstantCodeName( context, element )
-                        for element
-                        in
-                        constant_value
-                    )
+                    count,
+                    element_name,
+                    element_name
                 )
             )
 
         return
 
     if constant_type is list:
-        if constant_value == []:
-            emit( "%s = PyList_New( 0 );" % constant_identifier )
-        else:
-            length = len( constant_value )
-            addMakeListUse( length )
+        if attemptToMarshal(constant_identifier, constant_value, emit):
+            return
 
-            # Make elements earlier than list itself.
-            for element in constant_value:
-                _addConstantInitCode(
-                    emit                = emit,
-                    constant_type       = type( element ),
-                    constant_value      = element,
-                    constant_identifier = getConstantCodeName(
-                        context  = context,
-                        constant = element
-                    ),
-                    context             = context
-                )
+        emit(
+            "%s = PyList_New( %d );" % (
+                constant_identifier,
+                len(constant_value)
+            )
+        )
 
+        for count, element_value in enumerate(constant_value):
+            element_name = getConstantCodeName(
+                context  = context,
+                constant = element_value
+            )
+
+            _addConstantInitCode(
+                emit                = emit,
+                constant_type       = type(element_value),
+                constant_value      = element_value,
+                constant_identifier = element_name,
+                module_level        = module_level,
+                context             = context
+            )
+
+            # Do not take references, these won't be deleted ever.
             emit(
-                "%s = MAKE_LIST%d( %s );" % (
+                "PyList_SET_ITEM( %s, %d, %s ); Py_INCREF( %s );" % (
                     constant_identifier,
-                    length,
-                    ", ".join(
-                        getConstantCodeName(
-                            context  = context,
-                            constant = element
-                        )
-                        for element
-                        in
-                        constant_value
-                    )
+                    count,
+                    element_name,
+                    element_name
                 )
             )
 
         return
 
     if constant_type is set:
-        if constant_value == set():
-            emit( "%s = PySet_New( NULL );" % constant_identifier )
-        else:
-            length = len( constant_value )
-            addMakeSetUse( length )
+        if attemptToMarshal(constant_identifier, constant_value, emit):
+            return
 
-            # Make elements earlier than list itself.
-            for element in constant_value:
-                _addConstantInitCode(
-                    emit                = emit,
-                    constant_type       = type( element ),
-                    constant_value      = element,
-                    constant_identifier = getConstantCodeName(
-                        context  = context,
-                        constant = element
-                    ),
-                    context             = context
-                )
+        emit("%s = PySet_New( NULL );" % constant_identifier)
+
+        for element_value in constant_value:
+            element_name = getConstantCodeName(
+                context  = context,
+                constant = element_value
+            )
+
+            _addConstantInitCode(
+                emit                = emit,
+                constant_type       = type(element_value),
+                constant_value      = element_value,
+                constant_identifier = element_name,
+                module_level        = module_level,
+                context             = context
+            )
 
             emit(
-                "%s = MAKE_SET%d( %s );" % (
+                "PySet_Add( %s, %s );" % (
                     constant_identifier,
-                    length,
-                    ", ".join(
-                        getConstantCodeName(
-                            context  = context,
-                            constant = element
-                        )
-                        for element
-                        in
-                        constant_value
-                    )
+                    element_name
                 )
             )
 
         return
 
     if constant_type in (frozenset, complex, unicode, long, range):
-        emit( _getUnstreamCode( constant_value, constant_identifier ) )
+        if attemptToMarshal(constant_identifier, constant_value, emit):
+            return
 
-        return
+        emit(_getUnstreamCode(constant_value, constant_identifier))
 
-    if constant_value in constant_builtin_types:
         return
 
     assert False, ( type(constant_value), constant_value, constant_identifier )
 
-def _lengthKey(value):
-    return (
-        len(value[1]),
-        value[1]
-    )
-
-the_contained_constants = {}
 
 def getConstantsInitCode(context):
     # There are many cases for constants to be created in the most efficient
     # way, pylint: disable=R0912
 
-    statements = []
+    emit = SourceCodeCollector()
 
-    all_constants = the_contained_constants
-    all_constants.update(context.getConstants())
+    sorted_constants = sorted(
+        iterItems(context.getConstants()),
+        key = lambda k: (len(k), k )
+    )
 
-    def receiveStatement(statement):
-        assert statement is not None
-
-        if statement not in statements:
-            statements.append(statement)
-
-    for constant_value, constant_identifier in \
-          sorted(all_constants.items(), key = _lengthKey):
+    for constant_identifier, constant_value in sorted_constants:
         _addConstantInitCode(
-            emit                = receiveStatement,
-            constant_type       = type(constant_value.getConstant()),
-            constant_value      = constant_value.getConstant(),
+            emit                = emit,
+            constant_type       = type(constant_value),
+            constant_value      = constant_value,
             constant_identifier = constant_identifier,
+            module_level        = False,
             context             = context
         )
 
+    return emit.codes
+
+
+def getConstantsDeclCode(context):
+    # There are many cases for constants of different types.
+    # pylint: disable=R0912
+    statements = []
+
+    sorted_constants = sorted(
+        iterItems(context.getConstants()),
+        key = lambda k: (len(k), k )
+    )
+
+    for constant_identifier, constant_value in sorted_constants:
+        # Need not declare built-in types.
+        if constant_value in constant_builtin_types:
+            continue
+        if constant_value is None:
+            continue
+        if constant_value is False:
+            continue
+        if constant_value is True:
+            continue
+        if constant_value is Ellipsis:
+            continue
+
+        if context.getConstantUseCount(constant_identifier) != 1:
+            statements.append("PyObject *%s;" % constant_identifier)
+
     return statements
 
 
-class HashableConstant:
-    __slots__ = ["constant"]
-
-    def __init__(self, constant):
-        self.constant = constant
-
-    def getConstant(self):
-        return self.constant
-
-    def __hash__(self):
-        try:
-            # For Python3: range objects with same ranges give different hash
-            # values. It's not even funny, is it.
-            if type(self.constant) is range:
-                raise TypeError
-
-            return hash(self.constant)
-        except TypeError:
-            return 7
-
-    def __eq__(self, other):
-        assert isinstance(other, self.__class__)
-
-        return compareConstants(self.constant, other.constant)
-
-
-def getConstantsDeclCode(context, for_header):
-    # There are many cases for constants of different types.
-    # pylint: disable=R0912
-    statements = []
-    statements2 = []
-
-    constants = context.getConstants()
-
-    contained_constants = {}
-
-    def considerForDeferral(constant_value):
-        if constant_value is None:
-            return
-
-        if constant_value is False:
-            return
-
-        if constant_value is True:
-            return
-
-        if constant_value is Ellipsis:
-            return
-
-        constant_type = type(constant_value)
-
-        if constant_type is type:
-            return
-
-        key = HashableConstant(constant_value)
-
-        if key not in contained_constants:
-            constant_identifier = getConstantCodeName(context, constant_value)
-
-            contained_constants[key] = constant_identifier
-
-            if constant_type in (tuple, list, set, frozenset):
-                for element in constant_value:
-                    considerForDeferral(element)
-            elif constant_type is dict:
-                for key, value in iterItems(constant_value):
-                    considerForDeferral(key)
-                    considerForDeferral(value)
-
-
-    for constant_value, constant_identifier in \
-            sorted(constants.items(), key = _lengthKey):
-        constant_type = type(constant_value.getConstant())
-
-        # Need not declare built-in types.
-        if constant_type is type:
-            continue
-
-        declaration = "PyObject *%s;" % constant_identifier
-
-        if for_header:
-            declaration = "extern " + declaration
-        else:
-            if constant_type in (tuple, dict, list, set, frozenset):
-                considerForDeferral( constant_value.getConstant() )
-
-        statements.append(declaration)
-
-    for key, value in sorted(contained_constants.items(), key = _lengthKey):
-        if key not in constants:
-            declaration = "PyObject *%s;" % value
-
-            statements2.append(declaration)
-
-    if not for_header:
-        # Using global here, as this is really a singleton, in the form of a
-        # module, pylint: disable=W0603
-        global the_contained_constants
-        the_contained_constants = contained_constants
-
-    return statements, statements2
-
-def getConstantAccess(context, constant):
+def getConstantAccess(to_name, constant, emit, context):
     # Many cases, because for each type, we may copy or optimize by creating
-    # empty.  pylint: disable=R0911,R0912
+    # empty.  pylint: disable=R0911,R0912, R0915
 
     if type(constant) is dict:
         if constant:
@@ -562,38 +554,29 @@ def getConstantAccess(context, constant):
                 needs_deep = False
 
             if needs_deep:
-                return Identifier(
-                    "DEEP_COPY( %s )" % getConstantCode(
-                        constant = constant,
-                        context  = context
-                    ),
-                    1
-                )
-            else:
-                return Identifier(
-                    "PyDict_Copy( %s )" % getConstantCode(
-                        constant = constant,
-                        context  = context
-                    ),
-                    1
-                )
-
-        else:
-            return EmptyDictIdentifier()
-    elif type(constant) is set:
-        if constant:
-            return Identifier(
-                "PySet_New( %s )" % getConstantCode(
+                code = "DEEP_COPY( %s )" % getConstantCode(
                     constant = constant,
                     context  = context
-                ),
-                1
+                )
+            else:
+                code = "PyDict_Copy( %s )" % getConstantCode(
+                    constant = constant,
+                    context  = context
+                )
+        else:
+            code = "PyDict_New()"
+
+        ref_count = 1
+    elif type(constant) is set:
+        if constant:
+            code = "PySet_New( %s )" % getConstantCode(
+                constant = constant,
+                context  = context
             )
         else:
-            return Identifier(
-                "PySet_New( NULL )",
-                1
-            )
+            code = "PySet_New( NULL )"
+
+        ref_count = 1
     elif type(constant) is list:
         if constant:
             for value in constant:
@@ -604,26 +587,19 @@ def getConstantAccess(context, constant):
                 needs_deep = False
 
             if needs_deep:
-                return Identifier(
-                    "DEEP_COPY( %s )" % getConstantCode(
-                        constant = constant,
-                        context  = context
-                    ),
-                    1
+                code = "DEEP_COPY( %s )" % getConstantCode(
+                    constant = constant,
+                    context  = context
                 )
             else:
-                return Identifier(
-                    "LIST_COPY( %s )" % getConstantCode(
-                        constant = constant,
-                        context  = context
-                    ),
-                    1
+                code = "LIST_COPY( %s )" % getConstantCode(
+                    constant = constant,
+                    context  = context
                 )
         else:
-            return Identifier(
-                "PyList_New( 0 )",
-                1
-            )
+            code = "PyList_New( 0 )"
+
+        ref_count = 1
     elif type(constant) is tuple:
         for value in constant:
             if isMutable(value):
@@ -633,20 +609,114 @@ def getConstantAccess(context, constant):
             needs_deep = False
 
         if needs_deep:
-            return Identifier(
-                "DEEP_COPY( %s )" % getConstantCode(
-                    constant = constant,
-                    context  = context
-                ),
-                1
+            code = "DEEP_COPY( %s )" % getConstantCode(
+                 constant = constant,
+                 context  = context
             )
+
+            ref_count = 1
         else:
-            return getConstantHandle(
+            code = getConstantCode(
                 context  = context,
                 constant = constant
             )
+
+            ref_count = 0
     else:
-        return getConstantHandle(
+        code = getConstantCode(
             context  = context,
             constant = constant
         )
+
+        ref_count = 0
+
+    emit(
+        "%s = %s;" % (
+            to_name,
+            code,
+        )
+    )
+
+    if ref_count:
+        context.addCleanupTempName(to_name)
+
+
+def getModuleConstantCode(constant, context):
+    assert type(constant) is str
+
+    result = _getConstantInitValueCode(
+        context        = context,
+        constant_value = constant,
+        constant_type  = type(constant)
+    )
+
+    assert result is not None
+
+    return result
+
+
+constant_counts = {}
+
+def getConstantInitCodes(module_context):
+    decls = []
+    inits = []
+
+    sorted_constants = sorted(
+        module_context.getConstants(),
+        key = lambda k: (len(k), k )
+    )
+
+    global_context = module_context.global_context
+
+    for constant_identifier in sorted_constants:
+        if not constant_identifier.startswith("const_"):
+            continue
+
+        if global_context.getConstantUseCount(constant_identifier ) == 1:
+            qualifier = "static "
+
+            constant_value = global_context.constants[constant_identifier]
+
+            _addConstantInitCode(
+                emit                = inits.append,
+                constant_type       = type(constant_value),
+                constant_value      = constant_value,
+                constant_identifier = constant_identifier,
+                module_level        = True,
+                context             = module_context
+            )
+        else:
+            qualifier = "extern "
+
+        decls.append(qualifier + "PyObject *" + constant_identifier + ";")
+
+
+    return decls, inits
+
+
+def allocateNestedConstants(module_context):
+    def considerForDeferral(constant_value):
+        if isMarshalConstant(constant_value):
+            return
+
+        module_context.getConstantCode(constant_value)
+
+        constant_type = type(constant_value)
+
+        if constant_type in (tuple, list, set, frozenset):
+            for element in constant_value:
+                considerForDeferral(element)
+        elif constant_type is dict:
+            for key, value in iterItems(constant_value):
+                considerForDeferral(key)
+                considerForDeferral(value)
+
+    for constant_identifier in set(module_context.getConstants()):
+        constant_value = module_context.global_context.constants[
+            constant_identifier
+        ]
+
+        constant_type = type(constant_value)
+
+        if constant_type in (tuple, dict, list, set, frozenset):
+            considerForDeferral(constant_value)
