@@ -391,7 +391,7 @@ if ( found == false && %(parameter_name_object)s == key )
 """
 
 argparse_template_assign_from_dict_parameter_slow_path = """\
-if ( found == false && RICH_COMPARE_BOOL_EQ_PARAMETERS( %(parameter_name_object)s, key ) )
+if ( found == false && RICH_COMPARE_BOOL_EQ( %(parameter_name_object)s, key ) == 1 )
 {
 %(parameter_assign_from_kw)s
     found = true;
@@ -400,7 +400,7 @@ if ( found == false && RICH_COMPARE_BOOL_EQ_PARAMETERS( %(parameter_name_object)
 """
 
 argparse_template_assign_from_dict_parameter_slow_path_kw_only = """\
-if ( found == false && RICH_COMPARE_BOOL_EQ_PARAMETERS( %(parameter_name_object)s, key ) )
+if ( found == false && RICH_COMPARE_BOOL_EQ( %(parameter_name_object)s, key ) == 1 )
 {
 %(parameter_assign_from_kw)s
     found = true;
