@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,14 +18,20 @@
 """ Optimizations of builtins to builtin calls.
 
 """
-from nuitka.nodes.ParameterSpecs import ParameterSpec, TooManyArguments, matchCall
+import math
+import sys
 
+from nuitka.nodes.ParameterSpecs import (
+    ParameterSpec,
+    TooManyArguments,
+    matchCall
+)
 from nuitka.Utils import python_version
 
-import sys, math
 
-class BuiltinParameterSpec( ParameterSpec ):
-    def __init__( self, name, arg_names, default_count, list_star_arg = None, dict_star_arg = None ):
+class BuiltinParameterSpec(ParameterSpec):
+    def __init__( self, name, arg_names, default_count, list_star_arg = None,
+                  dict_star_arg = None ):
         ParameterSpec.__init__(
             self,
             name           = name,
@@ -38,28 +44,28 @@ class BuiltinParameterSpec( ParameterSpec ):
 
         self.builtin = __builtins__[ name ]
 
-    def __repr__( self ):
+    def __repr__(self):
         return "<BuiltinParameterSpec %s>" % self.name
 
-    def getName( self ):
+    def getName(self):
         return self.name
 
-    def isCompileTimeComputable( self, values ):
+    def isCompileTimeComputable(self, values):
         for value in values:
             if value is not None and not value.isCompileTimeConstant():
                 return False
         else:
             return True
 
-    def simulateCall( self, given_values ):
+    def simulateCall(self, given_values):
         # Using star dict call for simulation and catch any exception as really
         # fatal, pylint: disable=W0142,W0703
 
         try:
-            given_normal_args = given_values[ : len( self.normal_args ) ]
+            given_normal_args = given_values[:len(self.normal_args)]
 
             if self.list_star_arg:
-                given_list_star_args = given_values[ len( self.normal_args ) ]
+                given_list_star_args = given_values[len( self.normal_args )]
             else:
                 given_list_star_args = None
 
@@ -70,21 +76,22 @@ class BuiltinParameterSpec( ParameterSpec ):
 
             arg_dict = {}
 
-            for arg_name, given_value in zip( self.normal_args, given_normal_args ):
-                assert type( given_value ) not in ( tuple, list ), ( "do not like a tuple %s" % ( given_value, ))
+            for arg_name, given_value in zip(self.normal_args, given_normal_args):
+                assert type( given_value ) not in ( tuple, list ), \
+                  ( "do not like a tuple %s" % ( given_value, ))
 
                 if given_value is not None:
                     arg_dict[ arg_name ] = given_value.getCompileTimeConstant()
 
             if given_dict_star_args:
-                for given_dict_star_arg in reversed( given_dict_star_args ):
-                    arg_name = given_dict_star_arg.getKey()
-                    arg_value = given_dict_star_arg.getValue()
+                for given_dict_star_arg in reversed(given_dict_star_args):
+                    arg_name = given_dict_star_arg.getKey().getCompileTimeConstant()
+                    arg_value = given_dict_star_arg.getValue().getCompileTimeConstant()
 
-                    arg_dict[ arg_name.getCompileTimeConstant() ] = arg_value.getCompileTimeConstant()
+                    arg_dict[arg_name] = arg_value
 
         except Exception as e:
-            sys.exit( "Fatal problem: %r" % e )
+            sys.exit("Fatal problem: %r" % e)
 
         if given_list_star_args:
             return self.builtin(
@@ -92,15 +99,15 @@ class BuiltinParameterSpec( ParameterSpec ):
                 **arg_dict
             )
         else:
-            return self.builtin( **arg_dict )
+            return self.builtin(**arg_dict)
 
 
-class BuiltinParameterSpecNoKeywords( BuiltinParameterSpec ):
+class BuiltinParameterSpecNoKeywords(BuiltinParameterSpec):
 
-    def allowsKeywords( self ):
+    def allowsKeywords(self):
         return False
 
-    def simulateCall( self, given_values ):
+    def simulateCall(self, given_values):
         # Using star dict call for simulation and catch any exception as really fatal,
         # pylint: disable=W0142,W0703
 
@@ -133,8 +140,8 @@ class BuiltinParameterSpecNoKeywords( BuiltinParameterSpec ):
         return self.builtin( *arg_list )
 
 
-class BuiltinParameterSpecExceptions( BuiltinParameterSpec ):
-    def __init__( self, exception_name, default_count ):
+class BuiltinParameterSpecExceptions(BuiltinParameterSpec):
+    def __init__(self, exception_name, default_count):
         # TODO: Parameter default_count makes no sense for exceptions probably.
         BuiltinParameterSpec.__init__(
             self,
@@ -144,17 +151,17 @@ class BuiltinParameterSpecExceptions( BuiltinParameterSpec ):
             list_star_arg = "args"
         )
 
-    def allowsKeywords( self ):
+    def allowsKeywords(self):
         return False
 
-    def getKeywordRefusalText( self ):
+    def getKeywordRefusalText(self):
         return "exceptions.%s does not take keyword arguments" % self.name
 
-    def getCallableName( self ):
+    def getCallableName(self):
         return "exceptions." + self.getName()
 
 
-def makeBuiltinParameterSpec( exception_name ):
+def makeBuiltinParameterSpec(exception_name):
     if exception_name == "ImportError" and python_version >= 330:
         # TODO: Create this beast, needs keyword only arguments to be supported,
         # currently user of this function must take care to not have them.
@@ -166,11 +173,30 @@ def makeBuiltinParameterSpec( exception_name ):
     )
 
 builtin_int_spec = BuiltinParameterSpec( "int", ( "x", "base" ), 2 )
+
 # These builtins are only available for Python2
 if python_version < 300:
-    builtin_long_spec = BuiltinParameterSpec( "long", ( "x", "base" ), 2 )
-    builtin_execfile_spec = BuiltinParameterSpecNoKeywords( "execfile", ( "filename", "globals", "locals" ), 2 )
-    builtin_unicode_spec = BuiltinParameterSpec( "unicode", ( "string", "encoding", "errors" ), 3 )
+    builtin_long_spec = BuiltinParameterSpec(
+        "long",
+        ("x", "base"),
+        2
+    )
+    builtin_execfile_spec = BuiltinParameterSpecNoKeywords(
+        "execfile",
+        ("filename", "globals", "locals"),
+        2
+    )
+    builtin_unicode_spec = BuiltinParameterSpec(
+        "unicode",
+        ("string", "encoding", "errors"),
+        3
+    )
+    builtin_xrange_spec = BuiltinParameterSpec(
+        "xrange",
+        ("start", "stop", "step"),
+        2
+    )
+
 
 builtin_bool_spec = BuiltinParameterSpec( "bool", ( "x", ), 1 )
 builtin_float_spec = BuiltinParameterSpec( "float", ( "x", ), 1 )
@@ -189,7 +215,7 @@ builtin_list_spec = BuiltinParameterSpec( "list", ( "sequence", ), 1 )
 builtin_set_spec = BuiltinParameterSpecNoKeywords( "set", ( "iterable", ), 1 )
 
 builtin_import_spec = BuiltinParameterSpec( "__import__", ( "name", "globals", "locals", "fromlist", "level" ), 4 )
-builtin_open_spec = BuiltinParameterSpec( "open", ( "name", "mode", "buffering" ), 2 )
+builtin_open_spec = BuiltinParameterSpec( "open", ( "name", "mode", "buffering" ), 3 )
 builtin_chr_spec = BuiltinParameterSpecNoKeywords( "chr", ( "i", ), 0 )
 builtin_ord_spec = BuiltinParameterSpecNoKeywords( "ord", ( "c", ), 0 )
 builtin_bin_spec = BuiltinParameterSpecNoKeywords( "bin", ( "number", ), 0 )
@@ -203,11 +229,15 @@ builtin_vars_spec = BuiltinParameterSpecNoKeywords( "vars", ( "object", ), 1 )
 builtin_locals_spec = BuiltinParameterSpecNoKeywords( "locals", (), 0 )
 builtin_globals_spec = BuiltinParameterSpecNoKeywords( "globals", (), 0 )
 builtin_eval_spec = BuiltinParameterSpecNoKeywords( "eval", ( "source", "globals", "locals" ), 2 )
+if python_version < 300:
+    builtin_compile_spec = BuiltinParameterSpec("compile", ( "source", "filename", "mode", "flags", "dont_inherit" ), 2)
+else:
+    builtin_compile_spec = BuiltinParameterSpec("compile", ( "source", "filename", "mode", "flags", "dont_inherit", "optimize" ), 3)
 if python_version >= 300:
     builtin_exec_spec = BuiltinParameterSpecNoKeywords( "exec", ( "source", "globals", "locals" ), 2 )
 
-# Note: Iter in fact names its first argument if the default applies "collection", but it
-# won't matter much, fixed up in a wrapper.
+# Note: Iter in fact names its first argument if the default applies
+# "collection", fixed up in a wrapper.
 builtin_iter_spec = BuiltinParameterSpecNoKeywords( "iter", ( "callable", "sentinel" ), 1 )
 builtin_next_spec = BuiltinParameterSpecNoKeywords( "next", ( "iterator", "default" ), 1 )
 
@@ -224,24 +254,24 @@ builtin_setattr_spec = BuiltinParameterSpecNoKeywords( "setattr", ( "object", "n
 builtin_isinstance_spec = BuiltinParameterSpecNoKeywords( "isinstance", ( "instance", "cls" ), 0 )
 
 
-class BuiltinRangeSpec( BuiltinParameterSpecNoKeywords ):
-    def __init__( self, *args ):
+class BuiltinRangeSpec(BuiltinParameterSpecNoKeywords):
+    def __init__(self, *args):
         BuiltinParameterSpecNoKeywords.__init__( self, *args )
 
-    def isCompileTimeComputable( self, values ):
+    def isCompileTimeComputable(self, values):
         result = BuiltinParameterSpecNoKeywords.isCompileTimeComputable(
             self,
             values = values
         )
 
         if result:
-            arg_count = len( values )
+            arg_count = len(values)
 
             if arg_count == 1:
                 low = values[0]
 
-                # If it's not a number constant, we can compute the exception that will be
-                # raised.
+                # If it's not a number constant, we can compute the exception
+                # that will be raised.
                 if not low.isNumberConstant():
                     return True
 
@@ -249,8 +279,8 @@ class BuiltinRangeSpec( BuiltinParameterSpecNoKeywords ):
             elif arg_count == 2:
                 low, high = values
 
-                # If it's not a number constant, we can compute the exception that will be
-                # raised.
+                # If it's not a number constant, we can compute the exception
+                # that will be raised.
                 if not low.isNumberConstant() or not high.isNumberConstant():
                     return True
 
@@ -258,8 +288,9 @@ class BuiltinRangeSpec( BuiltinParameterSpecNoKeywords ):
             elif arg_count == 3:
                 low, high, step = values
 
-                if not low.isNumberConstant() or not high.isNumberConstant() or \
-                     not step.isNumberConstant():
+                if not low.isNumberConstant() or \
+                   not high.isNumberConstant() or \
+                   not step.isNumberConstant():
                     return True
 
                 low = low.getConstant()
@@ -286,16 +317,16 @@ class BuiltinRangeSpec( BuiltinParameterSpecNoKeywords ):
             return False
 
 
-builtin_range_spec = BuiltinRangeSpec( "range", ( "start", "stop", "step" ), 2 )
+builtin_range_spec = BuiltinRangeSpec("range", ( "start", "stop", "step" ), 2)
 
 
-def extractBuiltinArgs( node, builtin_spec, builtin_class, empty_special_class = None ):
+def extractBuiltinArgs(node, builtin_spec, builtin_class,
+                       empty_special_class = None):
     try:
         kw = node.getCallKw()
 
-
-        # TODO: Could check for too many / too few, even if they are unknown, we might
-        # raise that error, but that need not be optimized immediately.
+        # TODO: Could check for too many / too few, even if they are unknown, we
+        # might raise that error, but that need not be optimized immediately.
         if not kw.isMappingWithConstantStringKeys():
             return None
 
@@ -314,7 +345,7 @@ def extractBuiltinArgs( node, builtin_spec, builtin_class, empty_special_class =
         positional = args.getIterationValues()
 
         if not positional and not pairs and empty_special_class is not None:
-            return empty_special_class( source_ref = node.getSourceReference() )
+            return empty_special_class(source_ref = node.getSourceReference())
 
         args_dict = matchCall(
             func_name     = builtin_spec.getName(),
@@ -343,15 +374,16 @@ def extractBuiltinArgs( node, builtin_spec, builtin_class, empty_special_class =
     args_list = []
 
     for argument_name in builtin_spec.getArgumentNames():
-        args_list.append( args_dict[ argument_name ] )
+        args_list.append(args_dict[argument_name])
 
     if builtin_spec.getStarListArgumentName() is not None:
-        args_list.append( args_dict[ builtin_spec.getStarListArgumentName() ] )
+        args_list.append(args_dict[builtin_spec.getStarListArgumentName()])
 
     if builtin_spec.getStarDictArgumentName() is not None:
-        args_list.append( args_dict[ builtin_spec.getStarDictArgumentName() ] )
+        args_list.append(args_dict[builtin_spec.getStarDictArgumentName()])
 
-    # Using list reference for passing the arguments without names, pylint: disable=W0142
+    # Using list reference for passing the arguments without names,
+    # pylint: disable=W0142
     return builtin_class(
         *args_list,
         source_ref = node.getSourceReference()
