@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,78 +15,73 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Reformulation of classes
+""" Reformulation of class statements.
 
 Consult the developmer manual for information. TODO: Add ability to sync
 source code comments with developer manual sections.
 
 """
 
+from nuitka import Utils
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementDelVariable
+)
+from nuitka.nodes.AttributeNodes import (
+    ExpressionAttributeLookup,
+    ExpressionBuiltinHasattr
+)
+from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
+from nuitka.nodes.CallNodes import ExpressionCall, ExpressionCallNoKeywords
+from nuitka.nodes.ClassNodes import ExpressionSelectMetaclass
+from nuitka.nodes.ComparisonNodes import ExpressionComparison
+from nuitka.nodes.ConditionalNodes import (
+    ExpressionConditional,
+    StatementConditional
+)
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
+from nuitka.nodes.ContainerOperationNodes import (
+    ExpressionDictOperationGet,
+    StatementDictOperationRemove
+)
+from nuitka.nodes.FunctionNodes import (
+    ExpressionFunctionBody,
+    ExpressionFunctionCall,
+    ExpressionFunctionCreation,
+    ExpressionFunctionRef
+)
+from nuitka.nodes.GlobalsLocalsNodes import (
+    ExpressionBuiltinLocals,
+    StatementSetLocals
+)
+from nuitka.nodes.ParameterSpecs import ParameterSpec
+from nuitka.nodes.ReturnNodes import StatementReturn
+from nuitka.nodes.StatementNodes import StatementsSequence
+from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
+from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTargetTempVariableRef,
     ExpressionTargetVariableRef,
     ExpressionTempVariableRef,
     ExpressionVariableRef
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
-from nuitka.nodes.ComparisonNodes import ExpressionComparison
-
-from nuitka.nodes.CallNodes import (
-    ExpressionCallNoKeywords,
-    ExpressionCall
-)
-from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
-from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
-    ExpressionBuiltinHasattr
-)
-from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
-from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCreation,
-    ExpressionFunctionBody,
-    ExpressionFunctionCall,
-    ExpressionFunctionRef
-)
-from nuitka.nodes.ClassNodes import ExpressionSelectMetaclass
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
-from nuitka.nodes.ContainerOperationNodes import (
-    StatementDictOperationRemove,
-    ExpressionDictOperationGet
-)
-from nuitka.nodes.StatementNodes import StatementsSequence
-
-from nuitka.nodes.ConditionalNodes import (
-    ExpressionConditional,
-    StatementConditional
-)
-from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementDelVariable
-)
-
-from nuitka.nodes.GlobalsLocalsNodes import (
-    ExpressionBuiltinLocals,
-    StatementSetLocals
-)
-
-from nuitka.nodes.ParameterSpecs import ParameterSpec
 
 from .Helpers import (
-    makeStatementsSequenceFromStatement,
-    makeSequenceCreationOrConstant,
-    makeDictCreationOrConstant,
-    makeStatementsSequence,
+    buildNode,
+    buildNodeList,
     buildStatementsNode,
     extractDocFromBody,
-    buildNodeList,
-    buildNode,
-    getKind
+    getKind,
+    makeDictCreationOrConstant,
+    makeSequenceCreationOrConstant,
+    makeStatementsSequence,
+    makeStatementsSequenceFromStatement,
+    makeTryFinallyStatement,
+    popIndicatorVariable,
+    pushIndicatorVariable
 )
 
-from nuitka import Utils
-
 # TODO: Once we start to modify these, we should make sure, the copy is not
 # shared.
 make_class_parameters = ParameterSpec(
@@ -99,13 +94,13 @@ make_class_parameters = ParameterSpec(
 )
 
 
-def _buildClassNode3( provider, node, source_ref ):
+def _buildClassNode3(provider, node, source_ref):
     # Many variables, due to the huge re-formulation that is going on here,
     # which just has the complexity, pylint: disable=R0914
 
     # This function is the Python3 special case with special re-formulation as
     # according to developer manual.
-    class_statements, class_doc = extractDocFromBody( node )
+    class_statements, class_doc = extractDocFromBody(node)
 
     # We need a scope for the temporary variables, and they might be closured.
     temp_scope = provider.allocateTempScope(
@@ -150,7 +145,7 @@ def _buildClassNode3( provider, node, source_ref ):
         source_ref = source_ref
     )
 
-    # source_ref_orig = source_ref
+    source_ref_orig = source_ref
     source_ref = source_ref.atInternal()
 
     if body is not None:
@@ -160,7 +155,7 @@ def _buildClassNode3( provider, node, source_ref ):
     statements = [
         StatementSetLocals(
             new_locals = ExpressionTempVariableRef(
-                variable   = tmp_prepared.makeReference( provider ),
+                variable   = tmp_prepared.makeReference(provider),
                 source_ref = source_ref
             ),
             source_ref = source_ref
@@ -175,7 +170,7 @@ def _buildClassNode3( provider, node, source_ref ):
                 source_ref    = source_ref,
                 user_provided = True
             ),
-            source_ref   = source_ref.atInternal()
+            source_ref   = source_ref
         )
     ]
 
@@ -214,18 +209,18 @@ def _buildClassNode3( provider, node, source_ref ):
             )
         )
 
-    if Utils.python_version >= 340:
+    if Utils.python_version >= 340 and False: # TODO: Temporarily reverted:
         tmp_class = class_creation_function.allocateTempVariable(
             temp_scope = None,
             name       = "__class__"
         )
 
         class_target_variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_class.makeReference( class_creation_function ),
+            variable   = tmp_class.makeReference(class_creation_function),
             source_ref = source_ref
         )
         class_variable_ref = ExpressionTempVariableRef(
-            variable   = tmp_class.makeReference( class_creation_function ),
+            variable   = tmp_class.makeReference(class_creation_function),
             source_ref = source_ref
         )
     else:
@@ -244,7 +239,7 @@ def _buildClassNode3( provider, node, source_ref ):
             variable_ref = class_target_variable_ref,
             source       = ExpressionCall(
                 called     = ExpressionTempVariableRef(
-                    variable   = tmp_metaclass.makeReference( provider ),
+                    variable   = tmp_metaclass.makeReference(provider),
                     source_ref = source_ref
                 ),
                 args       = makeSequenceCreationOrConstant(
@@ -256,7 +251,7 @@ def _buildClassNode3( provider, node, source_ref ):
                             user_provided = True
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference( provider ),
+                            variable   = tmp_bases.makeReference(provider),
                             source_ref = source_ref
                         ),
                         ExpressionBuiltinLocals(
@@ -266,7 +261,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     source_ref    = source_ref
                 ),
                 kw         = ExpressionTempVariableRef(
-                    variable   = tmp_class_decl_dict.makeReference( provider ),
+                    variable   = tmp_class_decl_dict.makeReference(provider),
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
@@ -288,7 +283,7 @@ def _buildClassNode3( provider, node, source_ref ):
     # The class body is basically a function that implicitely, at the end
     # returns its locals and cannot have other return statements contained.
 
-    class_creation_function.setBody( body )
+    class_creation_function.setBody(body)
 
     # The class body is basically a function that implicitely, at the end
     # returns its created class and cannot have other return statements
@@ -317,7 +312,9 @@ def _buildClassNode3( provider, node, source_ref ):
         decorated_body = ExpressionCallNoKeywords(
             called     = decorator,
             args       = ExpressionMakeTuple(
-                elements   = ( decorated_body, ),
+                elements   = (
+                    decorated_body,
+                ),
                 source_ref = source_ref
             ),
             source_ref = decorator.getSourceReference()
@@ -340,7 +337,7 @@ def _buildClassNode3( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_decl_dict.makeReference( provider ),
+                variable   = tmp_class_decl_dict.makeReference(provider),
                 source_ref = source_ref
             ),
             source       = makeDictCreationOrConstant(
@@ -354,7 +351,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     node.keywords
                 ],
                 values = [
-                    buildNode( provider, keyword.value, source_ref )
+                    buildNode(provider, keyword.value, source_ref)
                     for keyword in
                     node.keywords
                 ],
@@ -365,7 +362,7 @@ def _buildClassNode3( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference( provider ),
+                variable   = tmp_metaclass.makeReference(provider),
                 source_ref = source_ref
             ),
             source       = ExpressionSelectMetaclass(
@@ -401,7 +398,7 @@ def _buildClassNode3( provider, node, source_ref ):
                     ),
                     no_expression  = ExpressionConditional(
                         condition      = ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference( provider ),
+                            variable   = tmp_bases.makeReference(provider),
                             source_ref = source_ref
                         ),
                         no_expression  = ExpressionBuiltinRef(
@@ -430,12 +427,12 @@ def _buildClassNode3( provider, node, source_ref ):
                     source_ref     = source_ref
                 ),
                 bases     = ExpressionTempVariableRef(
-                    variable   = tmp_bases.makeReference( provider ),
+                    variable   = tmp_bases.makeReference(provider),
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
-            source_ref = source_ref
+            source_ref = source_ref_orig
         ),
         StatementConditional(
             condition  = ExpressionComparison(
@@ -546,49 +543,52 @@ def _buildClassNode3( provider, node, source_ref ):
             source     = decorated_body,
             source_ref = source_ref
         ),
+    )
+
+    final = (
         StatementDelVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( provider ),
+                variable   = tmp_bases.makeReference(provider),
                 source_ref = source_ref
             ),
-            tolerant   = False,
+            tolerant   = True,
             source_ref = source_ref
         ),
         StatementDelVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_decl_dict.makeReference( provider ),
+                variable   = tmp_class_decl_dict.makeReference(provider),
                 source_ref = source_ref
             ),
-            tolerant   = False,
+            tolerant   = True,
             source_ref = source_ref
         ),
         StatementDelVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference( provider ),
+                variable   = tmp_metaclass.makeReference(provider),
                 source_ref = source_ref
             ),
-            tolerant   = False,
+            tolerant   = True,
             source_ref = source_ref
         ),
         StatementDelVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_prepared.makeReference( provider ),
+                variable   = tmp_prepared.makeReference(provider),
                 source_ref = source_ref
             ),
-            tolerant   = False,
+            tolerant   = True,
             source_ref = source_ref
         )
     )
 
-    return StatementsSequence(
-        statements = statements,
+    return makeTryFinallyStatement(
+        tried      = statements,
+        final      = final,
         source_ref = source_ref
     )
 
 
-
-def _buildClassNode2( provider, node, source_ref ):
-    class_statements, class_doc = extractDocFromBody( node )
+def _buildClassNode2(provider, node, source_ref):
+    class_statements, class_doc = extractDocFromBody(node)
 
     # This function is the Python2 special case with special re-formulation as
     # according to developer manual.
@@ -671,9 +671,9 @@ def _buildClassNode2( provider, node, source_ref ):
     temp_scope = provider.allocateTempScope( "class_creation" )
 
     tmp_bases = provider.allocateTempVariable( temp_scope, "bases" )
-    tmp_class_dict = provider.allocateTempVariable( temp_scope, "class_dict" )
-    tmp_metaclass = provider.allocateTempVariable( temp_scope, "metaclass" )
-    tmp_class = provider.allocateTempVariable( temp_scope, "class" )
+    tmp_class_dict = provider.allocateTempVariable(temp_scope, "class_dict")
+    tmp_metaclass = provider.allocateTempVariable(temp_scope, "metaclass")
+    tmp_class = provider.allocateTempVariable(temp_scope, "class")
 
     statements = [
         StatementAssignmentVariable(
@@ -692,7 +692,7 @@ def _buildClassNode2( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_dict.makeReference( provider ),
+                variable   = tmp_class_dict.makeReference(provider),
                 source_ref = source_ref
             ),
             source       =   ExpressionFunctionCall(
@@ -713,7 +713,7 @@ def _buildClassNode2( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference( provider ),
+                variable   = tmp_metaclass.makeReference(provider),
                 source_ref = source_ref
             ),
             source       = ExpressionConditional(
@@ -725,14 +725,14 @@ def _buildClassNode2( provider, node, source_ref ):
                         user_provided = True
                     ),
                     right      = ExpressionTempVariableRef(
-                        variable   = tmp_class_dict.makeReference( provider ),
+                        variable   = tmp_class_dict.makeReference(provider),
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
                 ),
                 yes_expression = ExpressionDictOperationGet(
                     dicte = ExpressionTempVariableRef(
-                        variable   = tmp_class_dict.makeReference( provider ),
+                        variable   = tmp_class_dict.makeReference(provider),
                         source_ref = source_ref
                     ),
                     key   = ExpressionConstantRef(
@@ -756,12 +756,12 @@ def _buildClassNode2( provider, node, source_ref ):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class.makeReference( provider ),
+                variable   = tmp_class.makeReference(provider),
                 source_ref = source_ref
             ),
             source     = ExpressionCallNoKeywords(
                 called         = ExpressionTempVariableRef(
-                    variable   = tmp_metaclass.makeReference( provider ),
+                    variable   = tmp_metaclass.makeReference(provider),
                     source_ref = source_ref
                 ),
                 args           = ExpressionMakeTuple(
@@ -788,30 +788,6 @@ def _buildClassNode2( provider, node, source_ref ):
             ),
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( provider ),
-                source_ref = source_ref
-            ),
-            tolerant   = False,
-            source_ref = source_ref
-        ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_dict.makeReference( provider ),
-                source_ref = source_ref
-            ),
-            tolerant   = False,
-            source_ref = source_ref
-        ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference( provider ),
-                source_ref = source_ref
-            ),
-            tolerant   = False,
-            source_ref = source_ref
-        )
     ]
 
     for decorator in buildNodeList(
@@ -822,7 +798,7 @@ def _buildClassNode2( provider, node, source_ref ):
         statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_class.makeReference( provider ),
+                    variable   = tmp_class.makeReference(provider),
                     source_ref = source_ref
                 ),
                 source       = ExpressionCallNoKeywords(
@@ -851,36 +827,67 @@ def _buildClassNode2( provider, node, source_ref ):
                 source_ref    = source_ref
             ),
             source     = ExpressionTempVariableRef(
-                variable   = tmp_class.makeReference( provider ),
+                variable   = tmp_class.makeReference(provider),
                 source_ref = source_ref
             ),
             source_ref = source_ref
         )
     )
 
-    statements.append(
+    final = (
         StatementDelVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class.makeReference( provider ),
+                variable   = tmp_class.makeReference(provider),
                 source_ref = source_ref
             ),
-            tolerant   = False,
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_bases.makeReference( provider ),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_class_dict.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementDelVariable(
+            variable_ref = ExpressionTargetTempVariableRef(
+                variable   = tmp_metaclass.makeReference(provider),
+                source_ref = source_ref
+            ),
+            tolerant   = True,
             source_ref = source_ref
         )
     )
 
-    return StatementsSequence(
-        statements = statements,
+    return makeTryFinallyStatement(
+        tried      = statements,
+        final      = final,
         source_ref = source_ref
     )
 
-def buildClassNode( provider, node, source_ref ):
+
+def buildClassNode(provider, node, source_ref):
     assert getKind( node ) == "ClassDef"
 
     # Python2 and Python3 are similar, but fundamentally different, so handle
     # them in dedicated code.
 
-    if Utils.python_version >= 300:
-        return _buildClassNode3( provider, node, source_ref )
-    else:
-        return _buildClassNode2( provider, node, source_ref )
+    pushIndicatorVariable(Ellipsis)
+
+    try:
+        if Utils.python_version >= 300:
+            return _buildClassNode3(provider, node, source_ref)
+        else:
+            return _buildClassNode2(provider, node, source_ref)
+    finally:
+        popIndicatorVariable()
